//+------------------------------------------------------------------+
//|                                           FXCorrelator.mq4       |
//|                                           Copyright  2011,      |
//|                                                                  |
//+------------------------------------------------------------------+
//
// Original version, including licensing terms can be found at:
// FXcorrelator.com 
// Do not resell or Copy
//
#define LONG_DIRECTION "LONG"
#define SHORT_DIRECTION "SHORT"

#property copyright "FXCorrelator.com"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 White
#property indicator_color4 Gray
#property indicator_color5 Yellow
#property indicator_color6 Turquoise
#property indicator_color7 Magenta
#property indicator_color8 Lime

#property indicator_level1 0.0

// Indicator Configuration
extern int PerAvr=20,Delta=3;
extern string SymbolSuffix="";

extern int SignalCandle = 0;
extern int TimeFrame = 0;
extern double Min_Gap = 0.0;                    //Adil
extern bool Dispaly_Values = true;
extern bool DirectionFilter = false;
extern bool ShowCross = true;
extern bool EmailCross = false;
extern bool AlertCross = false;
extern int Scale = 1000;

extern bool ShowEUR=true;
extern bool ShowGBP=true;
extern bool ShowAUD=true;
extern bool ShowCHF=true;
extern bool ShowJPY=true;
extern bool ShowNZD=true;
extern bool ShowCAD=true;
extern bool ShowUSD=true;

int    Disp_counter;
string Last_direction = "";
string Lines_Crossed = "";

// index buffers for drawing
double Idx[],Idx1[],Idx2[],Idx3[],Idx4[],Idx5[],Idx6[],Idx7[];

// currency rates for calculation
double EUR,GBP,AUD,CHF,JPY,NZD,CAD,USD,A1,A2,A3,A4,A5,A6,A7;

datetime _lastalertTime = 0;

// Currency names and colors
string Currencies[] = {"EUR","GBP","AUD","CHF","JPY","NZD","CAD","USD"};
int Colors[] = {indicator_color1, indicator_color2, indicator_color3, indicator_color4,
                indicator_color5, indicator_color6, indicator_color7, indicator_color8};

bool ShowCurrency[8];
string _ccy1 = "";
string _ccy2 = "";
// house keeping
string ShortName;
bool ObjectsCreated = false;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   string ccy1 = StringSubstr(Symbol(),0,3);
   string ccy2 = StringSubstr(Symbol(),3,3);
   //if this is a currency chart, get the currencies concerned!
   
   int count = 0;
   while((_ccy1 == "" || _ccy2 == "") && count < 8)
   {
      if (Currencies[count] == ccy1)
      {
         _ccy1 = ccy1;
      }
      else if (Currencies[count] == ccy2)
      {
         _ccy2 = ccy2;
      }
      count++;
   }   
   
   // set up indicator buffers
   for ( int i = 0; i < 8; i++ )
   {
      SetIndexStyle(i,DRAW_LINE,STYLE_SOLID,2);
      SetIndexLabel(i,Currencies[i]);
         
      if (_ccy1 != "" && _ccy2 != "")
      {
         ShowCurrency[i] = ((_ccy1 == Currencies[i] || _ccy2 == Currencies[i]));  
      }
      else
      {
         if ( i == 0 )
            ShowCurrency[i] = ShowEUR;
         else if ( i == 1 )
            ShowCurrency[i] = ShowGBP;
         else if ( i == 2 )
            ShowCurrency[i] = ShowAUD;
         else if ( i == 3 )
            ShowCurrency[i] = ShowCHF;
         else if ( i == 4 )
            ShowCurrency[i] = ShowJPY;
         else if ( i == 5 )
            ShowCurrency[i] = ShowNZD;
         else if ( i == 6 )
            ShowCurrency[i] = ShowCAD;
         else if ( i == 7 )
            ShowCurrency[i] = ShowUSD;   
      }
   }

   // silly MQL needs this part to be done manually
   SetIndexBuffer(0,Idx);
   SetIndexBuffer(1,Idx1);
   SetIndexBuffer(2,Idx2);
   SetIndexBuffer(3,Idx3);
   SetIndexBuffer(4,Idx4);
   SetIndexBuffer(5,Idx5);
   SetIndexBuffer(6,Idx6);
   SetIndexBuffer(7,Idx7);
   

   
   // Set name to be displayed
   ShortName = "FXCorrelator_8020("+PerAvr+"/"+Delta+")Values >>";
   IndicatorShortName(ShortName);
   
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   // delete all objects we created
  ObjectsDeleteAll(WindowFind(ShortName));
   
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
   {
      string label = ObjectName(i);
      string avgString = PerAvr;
      int length = 20 + StringLen(avgString);
      if(StringSubstr(label, 0, length) == StringConcatenate("FXStrengthCrossArrow",PerAvr))
      {
         ObjectDelete(label);   
      }      
   }  
  return(0);
}

//+------------------------------------------------------------------+
//| Create a single descriptive object at the given coordinates      |
//+------------------------------------------------------------------+
void CreateObject(string currency, int window, int x, int y, int col)
{
   int ret = ObjectCreate("obj_"+currency,OBJ_LABEL,window,0,0);
   ObjectSetText("obj_"+currency,currency,8);
   ObjectSet("obj_"+currency,OBJPROP_COLOR,col);
   ObjectSet("obj_"+currency,OBJPROP_XDISTANCE,x);
   ObjectSet("obj_"+currency,OBJPROP_YDISTANCE,y);
}

//+-------------------------------------------------------------------+
//| Create all text labels (colored names of the displayed currencies)|
//+-------------------------------------------------------------------+
void CreateObjects()
{
   // start and increment of X coordinates
   int xStart = 4;
   int xIncrement = 25;
   
   // start and increment of Y coordinates
   int yStart = 16;
   int yIncrement = 0;
   
   int window = WindowFind(ShortName);

   // create all labels
   for(int i = 0; i < 8; i++ )
   {
      CreateObject(Currencies[i], window, xStart, yStart, Colors[i]);
      xStart += xIncrement;
      yStart += yIncrement;
   }
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{  

   // only create objects here, window handle is not valid in init()  
   if ( !ObjectsCreated )
   {
      CreateObjects();
      ObjectsCreated = true;
   }
  
   int counted_bars=IndicatorCounted();
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   int limit=Bars-counted_bars;
   for(int i=limit; i>=0; i--)    
   {
      int pivot_bar = iBarShift( NULL, TimeFrame, Time[i]);
      A1=(iMA("EURUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar)-iMA("EURUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta))/1; 
      A2=(iMA("GBPUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar)-iMA("GBPUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta))/1; 
      A3=(iMA("AUDUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar)-iMA("AUDUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta))/1; 
      A4=(iMA("USDCHF"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta)-iMA("USDCHF"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar))/1;
      A5=(iMA("USDJPY"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta)-iMA("USDJPY"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar))/100; 
      A6=(iMA("NZDUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar)-iMA("NZDUSD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta))/1;
      A7=(iMA("USDCAD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar+Delta)-iMA("USDCAD"+SymbolSuffix,TimeFrame,PerAvr,0,MODE_LWMA,PRICE_CLOSE,pivot_bar))/1;
      
      USD=-(A1+A2+A3+A4+A5+A6+A7)/7;
      EUR=A1-(A2+A3+A4+A5+A6+A7)/6;
      GBP=A2-(A1+A3+A4+A5+A6+A7)/6;
      AUD=A3-(A1+A2+A4+A5+A6+A7)/6;     
      CHF=A4-(A1+A2+A3+A5+A6+A7)/6;  
      JPY=A5-(A1+A2+A3+A4+A6+A7)/6;
      NZD=A6-(A1+A2+A3+A4+A5+A7)/6;
      CAD=A7-(A1+A2+A3+A4+A5+A6)/6;

//      StringSubstr(USD);
//      Comment ("USD - ",DoubleToStr(USD,5));

      double value1 = 0;             
      double value2 = 0;
      double previousValue1 = 0;             
      double previousValue2 = 0;
      double Disp_Val_1, Disp_Val_2;      
      
      
      //now check each currency and see if we are showing it!      
      if ( ShowCurrency[0] )
      {
         Idx[i] = EUR * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[0])            
            {
               value1 = Idx[i + SignalCandle];
               previousValue1 = Idx[i + SignalCandle+1];
               Disp_Val_1 = Idx[i];            }
            else
            {
               value2 = Idx[i + SignalCandle];
               previousValue2 = Idx[i + SignalCandle+1];
               Disp_Val_2 = Idx[i];            }
         }
      }
         
      if ( ShowCurrency[1] )
      {
         Idx1[i] = GBP * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[1])
            {
               value1 = Idx1[i + SignalCandle];
               previousValue1 = Idx1[i + SignalCandle+1];
               Disp_Val_1 = Idx1[i];            }
            else
            {
               value2 = Idx1[i + SignalCandle];
               previousValue2 = Idx1[i + SignalCandle+1];         
               Disp_Val_2 = Idx1[i];            }
         }
      }
      if ( ShowCurrency[2] )
      {
        Idx2[i] = AUD * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[2])
            {
               value1 = Idx2[i + SignalCandle];
               previousValue1 = Idx2[i + SignalCandle+1];
               Disp_Val_1 = Idx2[i];            }
            else
            {
               value2 = Idx2[i + SignalCandle];
               previousValue2 = Idx2[i + SignalCandle+1];         
               Disp_Val_2 = Idx2[i];            }
         }
      }   
      if ( ShowCurrency[3] )
      {
         Idx3[i] = CHF * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[3])
            {
               value1 = Idx3[i + SignalCandle];
               previousValue1 = Idx3[i + SignalCandle+1];
               Disp_Val_1 = Idx3[i];
            }
            else
            {
               value2 = Idx3[i + SignalCandle];
               previousValue2 = Idx3[i + SignalCandle+1];         
               Disp_Val_2 = Idx3[i];
            }
         }
      }
      if ( ShowCurrency[4] )
      {
         Idx4[i] = JPY * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[4])
            {
               value1 = Idx4[i + SignalCandle];
               previousValue1 = Idx4[i + SignalCandle+1];
               Disp_Val_1 = Idx4[i];
            }
            else
            {
               value2 = Idx4[i + SignalCandle];
               previousValue2 = Idx4[i + SignalCandle+1];         
               Disp_Val_2 = Idx4[i];
            }
         }
      }
      if ( ShowCurrency[5] )
      {
         Idx5[i] = NZD * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[5])
            {
               value1 = Idx5[i + SignalCandle];
               previousValue1 = Idx5[i + SignalCandle+1];
               Disp_Val_1 = Idx5[i];
            }
            else
            {
               value2 = Idx5[i + SignalCandle];
               previousValue2 = Idx5[i + SignalCandle+1];         
               Disp_Val_2 = Idx5[i];
            }
         }
      }
      if ( ShowCurrency[6] )
      {
         Idx6[i] = CAD * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[6])
            {
               value1 = Idx6[i + SignalCandle];
               previousValue1 = Idx6[i + SignalCandle+1];
               Disp_Val_1 = Idx6[i];
            }
            else
            {
               value2 = Idx6[i + SignalCandle];
               previousValue2 = Idx6[i + SignalCandle+1];         
               Disp_Val_2 = Idx6[i];
            }
         }
      }
      if ( ShowCurrency[7] )
      {
         Idx7[i] = USD * Scale;
         if (_ccy1 != "" && _ccy2 != "")
         {
            if (StringSubstr(Symbol(),0,3) == Currencies[7])
            {
               value1 = Idx7[i + SignalCandle];
               previousValue1 = Idx7[i + SignalCandle+1];
               Disp_Val_1 = Idx7[i];
            }
            else
            {
               value2 = Idx7[i + SignalCandle];
               previousValue2 = Idx7[i + SignalCandle+1];         
               Disp_Val_2 = Idx7[i];            }
         }
      }

//-------------                     Adil Start

double Gap;                                        
bool Disp_arrow = true;

  Gap = MathAbs(Disp_Val_1 - Disp_Val_2);
   if (Disp_Val_1 > Disp_Val_2)
      {Gap = (Disp_Val_1 - Disp_Val_2);}

   if (Disp_Val_1 < Disp_Val_2)
      {Gap = (Disp_Val_2 - Disp_Val_1);}
      
   if ((Disp_Val_1 < 0 && Disp_Val_2 < 0) || (Disp_Val_1 > 0 && Disp_Val_2 > 0))
      {Disp_arrow = false;}   
//-------------                  //Adil End

      string direction = "";
      
      if (_ccy1 != "" && _ccy2 != "")
      {  
         if (DirectionFilter)
         {
            if (value1 > value2 && previousValue1 < previousValue2  && value1 > previousValue1 && value2 < previousValue2)
            {
               direction = LONG_DIRECTION;
               Lines_Crossed = "Y";
            }
            else if (value1 < value2 && previousValue1 > previousValue2 && value1 < previousValue1 && value2 > previousValue2)
            {
               direction = SHORT_DIRECTION;
               Lines_Crossed = "Y";
            }
         }
         else
         {   
            if (value1 > value2 && previousValue1 < previousValue2)
            {
               direction = LONG_DIRECTION;
               Lines_Crossed = "Y";
            }
            else if (value1 < value2 && previousValue1 > previousValue2)
            {
               direction = SHORT_DIRECTION;
               Lines_Crossed = "Y";
            }
//         Comment (direction);
         }
      }

//-------------                  //Adil Start - Used for testing

         if (Dispaly_Values) 
            {Comment (_ccy1," = ", DoubleToStr(Disp_Val_1,4), "  /-/  ",
                      _ccy2," = ", DoubleToStr(Disp_Val_2,4),
                      "  Gap = ", Gap, "    Dir - ", Last_direction);
            }
         else
            {Comment ("");}
            
//-------------                  //Adil End

      if ((ShowCross || AlertCross || EmailCross) && direction != "")
      {
         if (ShowCross)
//         {                                          // original
//            DrawArrow(direction, i,"",false);
//            Last_direction = direction;
//         }
//         { if (Lines_Crossed == "Y" && Gap >= Min_Gap && Disp_arrow)  // original End

            { if (Lines_Crossed == "Y" && Gap >= Min_Gap)      // Adil
               {DrawArrow(direction, i,"",false);              // Adil
                Last_direction = direction;                    // Adil
                Lines_Crossed = "N";                           // Adil
               }
            }
         if (AlertCross && i == 0 && Time[0] != _lastalertTime)
         {
            Alert(Symbol(), " ", Period(), "M ", direction, ", Bid = ", Bid);              
         }
         
         if (EmailCross && i == 0 && Time[0] != _lastalertTime)
         {
            string s = StringConcatenate(Symbol(), " ", Period(), "M ", direction, " Strength Cross, Bid = ", Bid);
            SendMail(s,s);
         }
         
         if ((EmailCross || AlertCross) && i == 0 && Time[0] != _lastalertTime)
         {
            _lastalertTime = Time[0];
         }
      }
   }

   return(0);
}

void DrawArrow(string direction, int shift, string message, bool displayAlert)
{
   string label = "FXStrengthCrossArrow" + PerAvr + direction + message + "(" + Time[shift] + ")";
   color arrowColour;
   double value = 0;

      double dist = iATR(NULL,0,20,shift);
      double arrowMargin = iMA(NULL,0,8,0,MODE_LWMA,PRICE_MEDIAN,shift);   
//   Comment ("Arrow Margin = ",arrowMargin);
   
   if (direction == LONG_DIRECTION)
   {
      arrowColour = LimeGreen;
      value = arrowMargin - dist;
   }
   else 
   if (direction == SHORT_DIRECTION)
   {
      arrowColour = Yellow;
      value = (arrowMargin + (dist*2));   
   }
   else
   {
      Print ("Error with Arrow Label: ", label);
      if (displayAlert)
      {
         Alert (Symbol(), " ", Period(), ": Error trying to Draw Arrow for direction [",direction,"]");
      }
   }
   ObjectDelete(label);
   ObjectCreate(label, OBJ_ARROW, 0, Time[shift], value);   
   ObjectSet(label, OBJPROP_COLOR, arrowColour);
   ObjectSet(label, OBJPROP_WIDTH, 3);
   
   if (direction == SHORT_DIRECTION)
   {
      ObjectSet(label, OBJPROP_ARROWCODE, 242);
   }
   else if (direction == LONG_DIRECTION)
   {
      ObjectSet(label, OBJPROP_ARROWCODE, 241);
   }
}